@import "CPArray.j"
@import "CPString.j"

@import "CPPredicate.j"

CPNotPredicateType = 0;
CPAndPredicateType = 1;
CPOrPredicateType  = 2;
   
var CPCompoundPredicateType;

@implementation CPCompoundPredicate : CPPredicate
{
  CPCompoundPredicateType _type;
  CPArray                 _predicates;

}

- (id)initWithType:(CPCompoundPredicateType)type subpredicates:(CPArray)predicates
{
   _type = type;
   _predicates = predicates;
   return self;
}

+ (CPPredicate)notPredicateWithSubpredicate:(CPPredicate)predicate
{
   return [[self alloc] initWithType:CPNotPredicateType subpredicates:[CPArray arrayWithObject:predicate]];
}

+ (CPPredicate)andPredicateWithSubpredicates:(CPArray *)predicates
{
   return [[self alloc] initWithType:CPAndPredicateType subpredicates:predicates];
}

+ (CPPredicate)orPredicateWithSubpredicates:(CPArray)predicates
{
   return [[self alloc] initWithType:CPOrPredicateType subpredicates:predicates];
}

- (CPPredicate)predicateWithSubstitutionVariables:(CPDictionary)variables
{
    var subp = [CPArray array],
        i;
        
    for (i =0; i < [subp count]; i++)
    {
        var p = [subp objectAtIndex:i];
        var sp = [p predicateWithSubstitutionVariables:variables];
        [subp addObject:sp];
    }
    return [[CPCompoundPredicate alloc] initWithType:_type subpredicates:subp];
}

- (CPString)predicateFormat
{
   var  result = [CPString string],
        args = [CPArray array],
        count = [_predicates count],
        i;
   
   for (i = 0;i < count;i++)
   {
    var subpredicate = [_predicates objectAtIndex:i],
        precedence = [subpredicate predicateFormat];
    
    if ([subpredicate isKindOfClass:[CPCompoundPredicate class]] &&
        [subpredicate compoundPredicateType] != _type)
      precedence = [CPString stringWithFormat:@"(%s)",precedence];
     
    [args addObject:precedence];
   }
   
   switch (_type)
   {
    case CPNotPredicateType:
     result = result + [CPString stringWithFormat:@"NOT %s",[args objectAtIndex:0]];
     break;
     
    case CPAndPredicateType:
     result = result + [CPString stringWithFormat:@"%s AND %s",[args objectAtIndex:0],[args objectAtIndex:1]];
     break;

    case CPOrPredicateType:
     result = result + [CPString stringWithFormat:@"%s OR %s",[args objectAtIndex:0],[args objectAtIndex:1]];
     break;
    }
   
   return result;
}

- (CPCompoundPredicateType)compoundPredicateType
{
   return _type;
}

- (CPArray)subpredicates
{
   return _predicates;
}

- (BOOL)evaluateWithObject:(id)object
{
   var  result = NO,
        count = [_predicates count],
        i;
   
   for (i = 0;i < count;i++)
   {
    var predicate = [_predicates objectAtIndex:i];
    
    switch (_type)
    {
     case CPNotPredicateType:
      return ![predicate evaluateWithObject:object];

     case CPAndPredicateType:
      if (i == 0)
       result = [predicate evaluateWithObject:object];
      else
       result = result && [predicate evaluateWithObject:object];
      break;
      
     case CPOrPredicateType:
      if ([predicate evaluateWithObject:object])
       return YES;
      break;
    }
   }
   
   return result;
}

@end
