
@import "CPString.j"
@import "CPArray.j"
@import "CPKeyValueCoding.j"
@import "CPDictionary.j"
@import "CPCoder.j"

CPConstantValueExpressionType = 0;
CPEvaluatedObjectExpressionType = 1;
CPVariableExpressionType = 2;
CPKeyPathExpressionType = 3;
CPFunctionExpressionType = 4;
CPAggregateExpressionType = 5;
CPSubqueryExpressionType = 6;
CPUnionSetExpressionType = 7;
CPIntersectSetExpressionType = 8;
CPMinusSetExpressionType = 9;

@implementation CPExpression : CPObject
{
    int _type;
}

- (id)initWithExpressionType:(int)type
{
   _type = type;
   return self;
}

- (id)initWithCoder:(CPCoder)coder
{
// IMPLEMENTED BY CONCRETE SUBCLASSES
}

- (void)encodeWithCoder:(CPCoder)coder
{
// IMPLEMENTED BY CONCRETE SUBCLASSES
}

+ (CPExpression)expressionForConstantValue:(id)value
{
   return [[CPExpression_constant alloc] initWithValue:value];
}

+ (CPExpression)expressionForEvaluatedObject
{
   return [[CPExpression_self alloc] init];
}

+ (CPExpression)expressionForVariable:(CPString)string
{
   return [[CPExpression_variable alloc] initWithVariable:string];
}

+ (CPExpression)expressionForKeyPath:(CPString)keyPath
{
   return [[CPExpression_keypath alloc] initWithKeyPath:keyPath];
}

+ (CPExpression)expressionForFunction:(CPString)function_name arguments:(CPArray)arguments
{

   var expression_function = [[CPExpression_function alloc] initWithName:function_name arguments:arguments];
   
   var selector = CPSelectorFromString(function_name);
   
   if (![expression_function respondsToSelector:selector])
       [CPException raise: CPInvalidArgumentException reason:@"Unknown function implementation: " + function_name];
  
  [expression_function setSelector:selector];
  
  return expression_function;
}

+ (CPExpression)expressionForAggregate:(CPArray)collection
{
    return [[CPExpression_aggregate alloc] initWithAggregate:collection];
}

+ (CPExpression)expressionForSubquery:(CPExpression)expression usingIteratorVariable:(CPString)variable predicate:(id)predicate
{
    return nil; // UNIMPLEMENTED
}

+ (CPExpression)expressionForUnionSet:(CPExpression)left with:(CPExpression)right
{
    return [[CPExpression_unionset alloc] initWithLeft:left right:right];
}

+ (CPExpression)expressionForIntersectSetSet:(CPExpression)left with:(CPExpression)right
{
    return [[CPExpression_intersectset alloc] initWithLeft:left right:right];
}

+ (CPExpression)expressionForMinusSet:(CPExpression)left with:(CPExpression)right
{
    return [[CPExpression_minusset alloc] initWithLeft:left right:right];
}

- (int)expressionType
{
   return _type;
}

- (id)constantValue
{
   [CPException raise:CPInvalidArgumentException reason:@"self is not of CPConstantValueExpressionType"];
   return nil;
}

- (CPString)variable
{
    [CPException raise:CPInvalidArgumentException reason:@"self is not of CPVariableExpressionType"];
    return nil;
}

- (CPString)keyPath
{
    [CPException raise:CPInvalidArgumentException reason:@"self is not of CPKeyPathExpressionType"];
    return nil;
}

- (CPString)function
{
    [CPException raise:CPInvalidArgumentException reason:@"self is not of CPFunctionExpressionType"];
    return nil;
}

- (CPArray)arguments
{
    [CPException raise:CPInvalidArgumentException reason:@"self is not of CPFunctionExpressionType"];
    return nil;
}

- (id)collection
{
    [CPException raise:CPInvalidArgumentException reason:@"self is not of CPAggregateExpressionType"];
    return nil;
}

- (id)expressionValueWithObject:(id)object context:(CPDictionary)context
{
// IMPLEMENTED BY SUBCLASSES
  return nil;
}

- (CPExpression)_expressionWithSubstitutionVariables:(CPDictionary)variables
{
// IMPLEMENTED BY SUBCLASSES
   return self;
}

@end

@import "CPExpression_constant.j"
@import "CPExpression_self.j"
@import "CPExpression_variable.j"
@import "CPExpression_keypath.j"
@import "CPExpression_function.j"
@import "CPExpression_aggregate.j"
@import "CPExpression_unionset.j"
@import "CPExpression_intersectset.j"
@import "CPExpression_minusset.j"
