
@import "CPValue.j"
@import "CPArray.j"
@import "CPNull.j"

@implementation CPPredicate : CPObject
{
}

- (CPString)description
{
   return [self predicateFormat];
}

+ (CPPredicate)predicateWithValue:(BOOL)value
{
   return [[CPPredicate_BOOL alloc] initWithBool:value];
}

- (BOOL) evaluateWithObject:(id)object
{
  // IMPLEMENTED BY SUBCLASSES
}

- (CPString) predicateFormat
{
  // IMPLEMENTED BY SUBCLASSES 
}

- (CPPredicate) predicateWithSubstitutionVariables: (CPDictionary)variables
{
    // IMPLEMENTED BY SUBCLASSES
}

+ (CPPredicate) predicateWithFormat: (CPString) format, ...
{
    // UNIMPLEMENTED
    return nil;
}

+ (CPPredicate) predicateWithFormat: (CPString)format argumentArray: (CPArray)args
{
    // UNIMPLEMENTED
    return nil;
}

+ (CPPredicate) predicateWithFormat: (CPString)format arguments: (va_list)args
{
    // UNIMPLEMENTED
    return nil;
}

@end

@implementation CPPredicate_BOOL : CPPredicate
{
    BOOL _value;
}

- (id)initWithBool:(BOOL)value
{
   _value = value;
   return self;
}

- (BOOL)evaluateObject:(id)object
{
   return _value;
}

- (CPString)predicateFormat
{
  return (_value) ? @"TRUEPREDICATE" : @"FALSEPREDICATE";
}

@end

@implementation CPArray (CPPredicate)

- (CPArray)filteredArrayUsingPredicate:(CPPredicate)predicate
{
    var count = [self count],
        result = [CPArray array],
        i;

    for (i = 0;i < count;i++)
    {
        var object = [self objectAtIndex:i];

        if ([predicate evaluateWithObject:object])
            [result addObject:object];
    }
    
    return result;
}

- (void)filterUsingPredicate:(CPPredicate)predicate
{
    var count = [self count];

    while (--count >= 0)
    {
        var check = [self objectAtIndex:count];

        if (![predicate evaluateWithObject:check])
            [self removeObjectAtIndex:count];
    }
}

@end


@import "CPCompoundPredicate.j"
@import "CPComparisonPredicate.j"
@import "CPExpression.j"

@import "CPExpression_operator.j"
@import "CPExpression_aggregate.j"
@import "CPExpression_assignment.j"
